@@ -1,4 +1,4 @@
-// Copyright 2019 Graham Clark. All rights reserved.  Use of this source
+// Copyright 2019-2022 Graham Clark. All rights reserved.  Use of this source
 // code is governed by the MIT license that can be found in the LICENSE
 // file.
 
@@ -13,7 +13,7 @@ import (
 	"sync"
 	"time"
 
-	"github.com/gdamore/tcell"
+	tcell "github.com/gdamore/tcell/v2"
 	log "github.com/sirupsen/logrus"
 )
 
@@ -82,22 +82,26 @@ type IApp interface {
 // provides services to a running gowid application, such as access to the
 // palette, the screen and the state of the mouse.
 type App struct {
-	IPalette                                 // App holds an IPalette and provides it to each widget when rendering
-	screen            tcell.Screen           // Each app has one screen
-	TCellEvents       chan tcell.Event       // Events from tcell e.g. resize
-	AfterRenderEvents chan IAfterRenderEvent // Functions intended to run on the widget goroutine
-	closing           bool                   // If true then app is in process of closing - it may be draining AfterRenderEvents.
-	closingMtx        sync.Mutex             // Make sure an AfterRenderEvent and closing don't race.
-	viewPlusMenus     IWidget                // The base widget that is displayed - includes registered menus
-	view              IWidget                // The base widget that is displayed under registered menus
-	colorMode         ColorMode              // The current color mode of the terminal - 256, 16, mono, etc
-	inCopyMode        bool                   // True if the app has been switched into "copy mode", for the user to copy a widget value
-	copyClaimed       int                    // True if a widget has "claimed" copy mode during this Render pass
-	copyClaimedBy     IIdentity
-	copyLevel         int
-	refreshCopy       bool
-	prevWasMouseMove  bool // True if we last processed simple mouse movement. We can optimize on slow
-	// systems by discarding subsequent mouse movement events.
+	IPalette                                    // App holds an IPalette and provides it to each widget when rendering
+	screen               tcell.Screen           // Each app has one screen
+	TCellEvents          chan tcell.Event       // Events from tcell e.g. resize
+	AfterRenderEvents    chan IAfterRenderEvent // Functions intended to run on the widget goroutine
+	closing              bool                   // If true then app is in process of closing - it may be draining AfterRenderEvents.
+	closingMtx           sync.Mutex             // Make sure an AfterRenderEvent and closing don't race.
+	viewPlusMenus        IWidget                // The base widget that is displayed - includes registered menus
+	view                 IWidget                // The base widget that is displayed under registered menus
+	colorMode            ColorMode              // The current color mode of the terminal - 256, 16, mono, etc
+	inCopyMode           bool                   // True if the app has been switched into "copy mode", for the user to copy a widget value
+	copyClaimed          int                    // True if a widget has "claimed" copy mode during this Render pass
+	copyClaimedBy        IIdentity
+	copyLevel            int
+	refreshCopy          bool
+	prevWasMouseMove     bool // True if we last processed simple mouse movement. We can optimize on slow
+	enableMouseMotion    bool
+	enableBracketedPaste bool
+	screenInited         bool
+	dontOwnScreen        bool
+	tty                  string
 
 	lastMouse    MouseState    // So I can tell if a button was previously clicked
 	MouseState                 // Track which mouse buttons are currently down
@@ -109,10 +113,14 @@ var _ IApp = (*App)(nil)
 
 // AppArgs is a helper struct, providing arguments for the initialization of App.
 type AppArgs struct {
-	View         IWidget
-	Palette      IPalette
-	Log          log.StdLogger
-	DontActivate bool
+	Screen               tcell.Screen
+	View                 IWidget
+	Palette              IPalette
+	EnableMouseMotion    bool
+	EnableBracketedPaste bool
+	Log                  log.StdLogger
+	DontActivate         bool
+	Tty                  string
 }
 
 // IUnhandledInput is used as a handler for application user input that is not handled by any
@@ -231,13 +239,17 @@ func NewApp(args AppArgs) (rapp *App, rerr error) {
 }
 
 // NewAppSafe returns an initialized App struct, or an error on failure. It will
-// initialize a tcell.Screen object behind the scenes, and enable mouse support
+// initialize a tcell.Screen object and enable mouse support if its not provided,
 // meaning that tcell will receive mouse events if the terminal supports them.
 func newApp(args AppArgs) (rapp *App, rerr error) {
-	screen, err := tcell.NewScreen()
-	if err != nil {
-		rerr = WithKVs(err, map[string]interface{}{"TERM": os.Getenv("TERM")})
-		return
+	screen := args.Screen
+	if screen == nil {
+		var err error
+		screen, err = tcellScreen(args.Tty)
+		if err != nil {
+			rerr = WithKVs(err, map[string]interface{}{"TERM": os.Getenv("TERM")})
+			return
+		}
 	}
 
 	var palette IPalette = args.Palette
@@ -263,23 +275,30 @@ func newApp(args AppArgs) (rapp *App, rerr error) {
 	}
 
 	res := &App{
-		IPalette:          palette,
-		screen:            screen,
-		TCellEvents:       tch,
-		AfterRenderEvents: wch,
-		closing:           false,
-		view:              args.View,
-		viewPlusMenus:     args.View,
-		colorMode:         Mode256Colors,
-		ClickTargets:      clicks,
-		log:               args.Log,
+		IPalette:             palette,
+		screen:               screen,
+		TCellEvents:          tch,
+		AfterRenderEvents:    wch,
+		closing:              false,
+		view:                 args.View,
+		viewPlusMenus:        args.View,
+		colorMode:            Mode256Colors,
+		ClickTargets:         clicks,
+		log:                  args.Log,
+		enableMouseMotion:    args.EnableMouseMotion,
+		enableBracketedPaste: args.EnableBracketedPaste,
+		dontOwnScreen:        args.Screen != nil,
+		tty:                  args.Tty,
 	}
 
-	if !args.DontActivate {
-		if err = res.initScreen(); err != nil {
+	if !res.dontOwnScreen && !args.DontActivate {
+		if err := res.initScreen(); err != nil {
 			return nil, err
 		}
 		res.initColorMode()
+		if res.enableBracketedPaste {
+			screen.EnablePaste()
+		}
 	}
 
 	screen.Clear()
@@ -461,7 +480,7 @@ func (a *App) Clips() []ICopyResult {
 // internal state, like the size of the underlying terminal.
 func (a *App) HandleTCellEvent(ev interface{}, unhandled IUnhandledInput) {
 	switch ev := ev.(type) {
-	case *tcell.EventKey:
+	case *tcell.EventKey, *tcell.EventPaste:
 		// This makes for a better experience on limited hardware like raspberry pi
 		debug.SetGCPercent(-1)
 		defer debug.SetGCPercent(100)
@@ -479,8 +498,12 @@ func (a *App) HandleTCellEvent(ev interface{}, unhandled IUnhandledInput) {
 			a.refreshCopy = false
 		}
 		a.RedrawTerminal()
+
+		//case *tcell.EventPaste:
+		//log.Infof("GCLA: app.go tcell paste")
+
 	case *tcell.EventMouse:
-		if !a.prevWasMouseMove || ev.Modifiers() != 0 || ev.Buttons() != 0 {
+		if !a.prevWasMouseMove || a.enableMouseMotion || ev.Modifiers() != 0 || ev.Buttons() != 0 {
 			switch ev.Buttons() {
 			case tcell.Button1:
 				a.MouseLeftClicked = true
@@ -660,7 +683,7 @@ Loop:
 // currentView's internal buffer is modified if currentView.Editable is true.
 func (a *App) handleInputEvent(ev interface{}, unhandled IUnhandledInput) {
 	switch ev.(type) {
-	case *tcell.EventKey, *tcell.EventMouse:
+	case *tcell.EventKey, *tcell.EventPaste, *tcell.EventMouse:
 		x, y := a.TerminalSize()
 		handled := UserInputIfSelectable(a.viewPlusMenus, ev, RenderBox{C: x, R: y}, Focused, a)
 		if !handled {
@@ -792,21 +815,30 @@ func (a *App) Quit() {
 // I can't make tcell claim and release the same screen successfully. Clients of
 // the app struct shouldn't cache the screen object returned via GetScreen().
 //
+// Assumes we own the screen...
 func (a *App) ActivateScreen() error {
-	screen, err := tcell.NewScreen()
+	screen, err := tcellScreen(a.tty)
 	if err != nil {
 		return WithKVs(err, map[string]interface{}{"TERM": os.Getenv("TERM")})
 	}
+	a.DeactivateScreen()
 	a.screen = screen
 	if err := a.initScreen(); err != nil {
 		return err
 	}
+	if a.enableBracketedPaste {
+		a.screen.EnablePaste()
+	}
 	return nil
 }
 
+// Assumes we own the screen
 func (a *App) DeactivateScreen() {
-	a.screen.Fini()
-	a.screen = nil
+	if a.screen != nil && a.screenInited {
+		a.screen.Fini()
+		a.screen = nil
+		a.screenInited = false
+	}
 }
 
 func (a *App) initScreen() error {
@@ -814,6 +846,7 @@ func (a *App) initScreen() error {
 		return WithKVs(err, map[string]interface{}{"TERM": os.Getenv("TERM")})
 	}
 
+	a.screenInited = true
 	a.initColorMode()
 
 	defFg := ColorDefault
@@ -825,13 +858,15 @@ func (a *App) initScreen() error {
 		defBg = IColorToTCell(bgCol, defBg, a.GetColorMode())
 		defSt = defSt.MergeUnder(style)
 	}
-	defStyle := tcell.Style(defSt.OnOff).Background(defBg.ToTCell()).Foreground(defFg.ToTCell())
+	defStyle := tcell.Style{}.Attributes(defSt.OnOff).Background(defBg.ToTCell()).Foreground(defFg.ToTCell())
 	// Ask TCell to set the screen's default style according to the palette's "default"
 	// config, if one is provided. This might make every screen cell underlined, for example,
 	// in the absence of overriding styling from widgets.
 	a.screen.SetStyle(defStyle)
 	a.screen.EnableMouse()
-
+	if a.enableBracketedPaste {
+		a.screen.EnablePaste()
+	}
 	return nil
 }
 
