@@ -1,4 +1,4 @@
-// Copyright 2019 Graham Clark. All rights reserved.  Use of this source
+// Copyright 2019-2022 Graham Clark. All rights reserved.  Use of this source
 // code is governed by the MIT license that can be found in the LICENSE
 // file.
 
@@ -13,7 +13,7 @@ import (
 	"github.com/gcla/gowid/gwutil"
 	"github.com/gcla/gowid/vim"
 	"github.com/gcla/gowid/widgets/fill"
-	"github.com/gdamore/tcell"
+	tcell "github.com/gdamore/tcell/v2"
 )
 
 //======================================================================
@@ -33,10 +33,12 @@ type IWidget interface {
 }
 
 type Widget struct {
-	widgets []gowid.IContainerWidget
-	focus   int // -1 means nothing selectable
-	prefCol int // caches the last set prefered col. Passes it on if widget hasn't changed focus
-	opt     Options
+	widgets      []gowid.IContainerWidget
+	focus        int    // -1 means nothing selectable
+	prefCol      int    // caches the last set prefered col. Passes it on if widget hasn't changed focus
+	widthHelper  []bool // optimizations to save frequent array allocations during use
+	widthHelper2 []bool
+	opt          Options
 	*gowid.Callbacks
 	gowid.AddressProvidesID
 	gowid.SubWidgetsCallbacks
@@ -67,10 +69,12 @@ func New(widgets []gowid.IContainerWidget, opts ...Options) *Widget {
 		opt.RightKeys = vim.AllRightKeys
 	}
 	res := &Widget{
-		widgets: widgets,
-		focus:   -1,
-		prefCol: -1,
-		opt:     opt,
+		widgets:      widgets,
+		focus:        -1,
+		prefCol:      -1,
+		widthHelper:  make([]bool, len(widgets)),
+		widthHelper2: make([]bool, len(widgets)),
+		opt:          opt,
 	}
 	res.SubWidgetsCallbacks = gowid.SubWidgetsCallbacks{CB: &res.Callbacks}
 	res.FocusCallbacks = gowid.FocusCallbacks{CB: &res.Callbacks}
@@ -159,6 +163,8 @@ func (w *Widget) SetSubWidgets(widgets []gowid.IWidget, app gowid.IApp) {
 			ws[i] = &gowid.ContainerWidget{IWidget: iw, D: gowid.RenderFlow{}}
 		}
 	}
+	w.widthHelper = make([]bool, len(widgets))
+	w.widthHelper2 = make([]bool, len(widgets))
 	oldFocus := w.Focus()
 	w.widgets = ws
 	w.SetFocus(app, oldFocus)
@@ -270,6 +276,16 @@ func (w *Widget) KeyIsRight(evk *tcell.EventKey) bool {
 	return vim.KeyIn(evk, w.opt.RightKeys)
 }
 
+type IWidthHelper interface {
+	WidthHelpers() ([]bool, []bool)
+}
+
+var _ IWidthHelper = (*Widget)(nil)
+
+func (w *Widget) WidthHelpers() ([]bool, []bool) {
+	return w.widthHelper, w.widthHelper2
+}
+
 //''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
 
 func SubWidgetSize(size gowid.IRenderSize, newX int, dim gowid.IWidgetDimension) gowid.IRenderSize {
@@ -311,7 +327,7 @@ func SubWidgetSize(size gowid.IRenderSize, newX int, dim gowid.IWidgetDimension)
 }
 
 func UserInput(w IWidget, ev interface{}, size gowid.IRenderSize, focus gowid.Selector, app gowid.IApp) bool {
-	res := true
+	res := false
 	subfocus := w.Focus()
 
 	subSizes := w.WidgetWidths(size, focus, subfocus, app)
@@ -322,6 +338,7 @@ func UserInput(w IWidget, ev interface{}, size gowid.IRenderSize, focus gowid.Se
 	forChild := false
 
 	if subfocus != -1 {
+		res = true
 		if evm, ok := ev.(*tcell.EventMouse); ok {
 			curX := 0
 			mx, _ := evm.Position()
@@ -418,7 +435,21 @@ func widgetWidthsExt(w gowid.ISelectChild, subs []gowid.IWidget, dims []gowid.IW
 	lenw := len(subs)
 
 	res := make([]int, lenw)
-	helper := make([]bool, lenw)
+	var widthHelper []bool
+	var widthHelper2 []bool
+	if w, ok := w.(IWidthHelper); ok {
+		// Save some allocations
+		widthHelper, widthHelper2 = w.WidthHelpers()
+		defer func() {
+			for i := 0; i < len(widthHelper); i++ {
+				widthHelper[i] = false
+				widthHelper2[i] = false
+			}
+		}()
+	} else {
+		widthHelper = make([]bool, lenw)
+		widthHelper2 = make([]bool, lenw)
+	}
 
 	haveColsTotal := false
 	var colsTotal int
@@ -451,22 +482,26 @@ func widgetWidthsExt(w gowid.ISelectChild, subs []gowid.IWidget, dims []gowid.IW
 			res[i] = c.BoxColumns()
 			trunc(&res[i])
 			colsUsed += res[i]
-			helper[i] = true
+			widthHelper[i] = true
+			widthHelper2[i] = true
 		case gowid.IRenderBox:
 			res[i] = w2.BoxColumns()
 			trunc(&res[i])
 			colsUsed += res[i]
-			helper[i] = true
+			widthHelper[i] = true
+			widthHelper2[i] = true
 		case gowid.IRenderFlowWith:
 			res[i] = w2.FlowColumns()
 			trunc(&res[i])
 			colsUsed += res[i]
-			helper[i] = true
+			widthHelper[i] = true
+			widthHelper2[i] = true
 		case gowid.IRenderWithUnits:
 			res[i] = w2.Units()
 			trunc(&res[i])
 			colsUsed += res[i]
-			helper[i] = true
+			widthHelper[i] = true
+			widthHelper2[i] = true
 		case gowid.IRenderRelative:
 			cols, ok := size.(gowid.IColumns)
 			if !ok {
@@ -475,11 +510,13 @@ func widgetWidthsExt(w gowid.ISelectChild, subs []gowid.IWidget, dims []gowid.IW
 			res[i] = int((w2.Relative() * float64(cols.Columns())) + 0.5)
 			trunc(&res[i])
 			colsUsed += res[i]
-			helper[i] = true
+			widthHelper[i] = true
+			widthHelper2[i] = true
 		case gowid.IRenderWithWeight:
 			// widget must be weighted
 			totalWeight += w2.Weight()
-			helper[i] = false
+			widthHelper[i] = false
+			widthHelper2[i] = false
 		default:
 			panic(gowid.DimensionError{Size: size, Dim: w2})
 		}
@@ -494,6 +531,7 @@ func widgetWidthsExt(w gowid.ISelectChild, subs []gowid.IWidget, dims []gowid.IW
 
 	// Now, divide up the remaining space among the weight columns
 	lasti := -1
+	maxedOut := false
 	for {
 		if colsLeft == 0 {
 			break
@@ -501,19 +539,21 @@ func widgetWidthsExt(w gowid.ISelectChild, subs []gowid.IWidget, dims []gowid.IW
 		doneone := false
 		totalWeight = 0
 		for i := 0; i < lenw; i++ {
-			if w2, ok := dims[i].(gowid.IRenderWithWeight); ok && !helper[i] {
+			if w2, ok := dims[i].(gowid.IRenderWithWeight); ok && !widthHelper[i] {
 				totalWeight += w2.Weight()
 			}
 		}
 		colsToDivideUp = colsLeft
 		for i := 0; i < lenw; i++ {
 			// Can only be weight here if !helper[i] ; but not sufficient for it to be eligible
-			if !helper[i] {
+			if !widthHelper[i] {
 				cols := int(((float32(dims[i].(gowid.IRenderWithWeight).Weight()) / float32(totalWeight)) * float32(colsToDivideUp)) + 0.5)
-				if max, ok := dims[i].(gowid.IRenderMaxUnits); ok {
-					if cols > max.MaxUnits() {
-						cols = max.MaxUnits()
-						helper[i] = true // this one is done
+				if !maxedOut {
+					if max, ok := dims[i].(gowid.IRenderMaxUnits); ok {
+						if cols >= max.MaxUnits() {
+							cols = max.MaxUnits()
+							widthHelper[i] = true // this one is done
+						}
 					}
 				}
 				if cols > colsLeft {
@@ -525,13 +565,25 @@ func widgetWidthsExt(w gowid.ISelectChild, subs []gowid.IWidget, dims []gowid.IW
 					}
 					res[i] += cols
 					colsLeft -= cols
-					lasti = i
+					lasti = gwutil.Max(i, lasti)
 					doneone = true
 				}
 			}
 		}
 		if !doneone {
-			break
+			// We used up all our extra space, after all weighted columns were maxed out. So
+			// we're done. Any extra space (should be just 1 unit at most) goes on the last columns.
+			if maxedOut {
+				break
+			}
+			// All the weighted columns have been assigned, and all were maxed out. We still
+			// have space to assign. So now grow the weighted columns, even though they don't need
+			// any more space for a full render - what else to do with the space!
+			maxedOut = true
+			// Reset; all false indices will be indices of weighted columns again
+			for i := 0; i < len(widthHelper); i++ {
+				widthHelper[i] = widthHelper2[i]
+			}
 		}
 	}
 	if lasti != -1 && colsLeft > 0 {
