@@ -1,4 +1,4 @@
-// Copyright 2019 Graham Clark. All rights reserved.  Use of this source
+// Copyright 2019-2022 Graham Clark. All rights reserved.  Use of this source
 // code is governed by the MIT license that can be found in the LICENSE
 // file.
 
@@ -9,13 +9,14 @@ import (
 	"fmt"
 	"strconv"
 
+	"github.com/araddon/dateparse"
 	"github.com/gcla/gowid"
 	"github.com/gcla/gowid/gwutil"
 	"github.com/gcla/gowid/widgets/columns"
 	"github.com/gcla/gowid/widgets/isselected"
 	"github.com/gcla/gowid/widgets/list"
 	"github.com/gcla/gowid/widgets/pile"
-	"github.com/gdamore/tcell"
+	tcell "github.com/gdamore/tcell/v2"
 	lru "github.com/hashicorp/golang-lru"
 )
 
@@ -110,6 +111,22 @@ func (s FloatCompare) Less(i, j string) bool {
 
 var _ ICompare = FloatCompare{}
 
+// DateTimeCompare is a unit type that satisfies ICompare, and can be used
+// for numerically comparing date/time values.
+type DateTimeCompare struct{}
+
+func (s DateTimeCompare) Less(i, j string) bool {
+	x, err1 := dateparse.ParseAny(i)
+	y, err2 := dateparse.ParseAny(j)
+	if err1 == nil && err2 == nil {
+		return x.Before(y)
+	} else {
+		return false
+	}
+}
+
+var _ ICompare = DateTimeCompare{}
+
 //======================================================================
 
 // ListWithPreferedColumn acts like a list.Widget but also satisfies gowid.IPreferedPosition.
