@@ -1,4 +1,4 @@
-// Copyright 2019 Graham Clark. All rights reserved.  Use of this source
+// Copyright 2019-2022 Graham Clark. All rights reserved.  Use of this source
 // code is governed by the MIT license that can be found in the LICENSE
 // file.
 
@@ -17,7 +17,7 @@ import (
 
 	"github.com/gcla/gowid"
 	"github.com/gcla/gowid/gwutil"
-	"github.com/gdamore/tcell"
+	tcell "github.com/gdamore/tcell/v2"
 	"github.com/mattn/go-runewidth"
 	log "github.com/sirupsen/logrus"
 	"golang.org/x/text/encoding/charmap"
@@ -311,6 +311,19 @@ func NewViewPort(c *gowid.Canvas, offset, height int) *ViewPortCanvas {
 	return res
 }
 
+func (c *ViewPortCanvas) Duplicate() gowid.ICanvas {
+	res := &ViewPortCanvas{
+		Canvas: c.Canvas.Duplicate().(*gowid.Canvas),
+		Offset: c.Offset,
+		Height: c.Height,
+	}
+	return res
+}
+
+func (c *ViewPortCanvas) MergeUnder(c2 gowid.IMergeCanvas, leftOffset, topOffset int, bottomGetsCursor bool) {
+	c.Canvas.MergeUnder(c2, leftOffset, topOffset+c.Offset, bottomGetsCursor)
+}
+
 func (v *ViewPortCanvas) BoxRows() int {
 	return v.Height
 }
@@ -417,6 +430,33 @@ func (c *Canvas) Write(p []byte) (n int, err error) {
 	return len(p), nil
 }
 
+func (c *Canvas) Duplicate() gowid.ICanvas {
+	res := &Canvas{}
+	*res = *c
+	res.ViewPortCanvas = c.ViewPortCanvas.Duplicate().(*ViewPortCanvas)
+	res.savedstyles = make(map[string]bool)
+	for k, v := range c.savedstyles {
+		res.savedstyles[k] = v
+	}
+	res.styles = make(map[string]bool)
+	for k, v := range c.styles {
+		res.styles[k] = v
+	}
+	res.tabstops = make([]int, len(c.tabstops))
+	for i, v := range res.tabstops {
+		res.tabstops[i] = v
+	}
+	res.escbuf = make([]byte, len(c.escbuf))
+	for i, v := range res.escbuf {
+		res.escbuf[i] = v
+	}
+	res.utf8Buffer = make([]byte, len(c.utf8Buffer))
+	for i, v := range res.utf8Buffer {
+		res.utf8Buffer[i] = v
+	}
+	return res
+}
+
 func (c *Canvas) Reset() {
 	c.alternateActive = false
 	c.escbuf = make([]byte, 0)
@@ -1187,7 +1227,7 @@ func (c *Canvas) Resize(width, height int) {
 	c.Offset -= height - c.Height
 	c.Height = height
 	if c.Height > c.Canvas.BoxRows() {
-		c.Height = c.Canvas.BoxRows()
+		c.Canvas.AppendBelow(gowid.NewCanvasOfSize(width, c.Height-c.Canvas.BoxRows()), false, false)
 	} else if c.Height < 1 {
 		c.Height = 1
 	}
@@ -1256,10 +1296,10 @@ func (c *Canvas) SetRune(r rune) {
 func (c *Canvas) MakeCellFrom(r rune) gowid.Cell {
 	var cell gowid.Cell = gowid.MakeCell(r, gowid.MakeTCellColorExt(tcell.ColorDefault), gowid.MakeTCellColorExt(tcell.ColorDefault), gowid.StyleNone)
 	if !c.fg.IsNone() {
-		cell = cell.WithForegroundColor(gowid.MakeTCellColorExt(tcell.Color(c.fg.Val() - 1)))
+		cell = cell.WithForegroundColor(gowid.MakeTCellColorExt(tcell.Color(c.fg.Val() - 1) + tcell.ColorValid))
 	}
 	if !c.bg.IsNone() {
-		cell = cell.WithBackgroundColor(gowid.MakeTCellColorExt(tcell.Color(c.bg.Val() - 1)))
+		cell = cell.WithBackgroundColor(gowid.MakeTCellColorExt(tcell.Color(c.bg.Val() - 1) + tcell.ColorValid))
 	}
 	if len(c.styles) > 0 {
 		for k, _ := range c.styles {
