@@ -1,4 +1,4 @@
-// Copyright 2019 Graham Clark. All rights reserved.  Use of this source
+// Copyright 2019-2022 Graham Clark. All rights reserved.  Use of this source
 // code is governed by the MIT license that can be found in the LICENSE
 // file.
 
@@ -17,12 +17,16 @@ import (
 	"time"
 	"unsafe"
 
+	"github.com/creack/pty"
 	"github.com/gcla/gowid"
 	"github.com/gcla/gowid/gwutil"
-	"github.com/gdamore/tcell"
-	"github.com/gdamore/tcell/terminfo"
-	"github.com/gdamore/tcell/terminfo/dynamic"
-	"github.com/kr/pty"
+	"github.com/gcla/gowid/widgets/columns"
+	"github.com/gcla/gowid/widgets/holder"
+	"github.com/gcla/gowid/widgets/null"
+	"github.com/gcla/gowid/widgets/vscroll"
+	tcell "github.com/gdamore/tcell/v2"
+	"github.com/gdamore/tcell/v2/terminfo"
+	"github.com/gdamore/tcell/v2/terminfo/dynamic"
 	log "github.com/sirupsen/logrus"
 )
 
@@ -54,6 +58,9 @@ type IWidget interface {
 	// IHotKeyPersistence determines how long a press of the hotkey will be in effect before
 	// keyboard user input is sent back to the underlying terminal.
 	IHotKeyPersistence
+	// IPaste tracks whether the paste start sequence has been seen wthout a matching
+	// paste end sequence
+	IPaste
 	// HotKeyActive returns true if the hotkey is currently in effect.
 	HotKeyActive() bool
 	// SetHotKeyActive sets the state of HotKeyActive.
@@ -68,6 +75,17 @@ type IWidget interface {
 	Scrolling() bool
 }
 
+type IHotKeyFunctions interface {
+	// Customized handling of hotkey sequences
+	HotKeyFunctions() []HotKeyInputFn
+}
+
+type IScrollbar interface {
+	ScrollbarEnabled() bool
+	EnableScrollbar(app gowid.IApp)
+	DisableScrollbar(app gowid.IApp)
+}
+
 type IHotKeyPersistence interface {
 	HotKeyDuration() time.Duration
 }
@@ -76,6 +94,12 @@ type IHotKeyProvider interface {
 	HotKey() tcell.Key
 }
 
+type IPaste interface {
+	PasteState(...bool) bool
+}
+
+type HotKeyInputFn func(ev *tcell.EventKey, w IWidget, app gowid.IApp) bool
+
 type HotKeyDuration struct {
 	D time.Duration
 }
@@ -97,17 +121,24 @@ type Bell struct{}
 type LEDs struct{}
 type Title struct{}
 type ProcessExited struct{}
+type HotKeyCB struct{}
 
 type bell struct{}
 type leds struct{}
 type title struct{}
+type hotkey struct{}
 
 type Options struct {
-	Command           []string
-	Env               []string
-	HotKey            IHotKeyProvider
-	HotKeyPersistence IHotKeyPersistence // the period of time a hotKey sticks after the first post-hotKey keypress
-	Scrollback        int
+	Command                 []string
+	Env                     []string
+	HotKey                  IHotKeyProvider
+	HotKeyPersistence       IHotKeyPersistence // the period of time a hotKey sticks after the first post-hotKey keypress
+	Scrollback              int
+	Scrollbar               bool            // disabled regardless of setting if there is no scrollback
+	HotKeyFns               []HotKeyInputFn // allow custom behavior after pressing the hotkey
+	EnableBracketedPaste    bool
+	KeyPressToEndScrollMode bool // set to true to enable legacy behavior - when the user has scrolled
+	// back to the prompt, still require a keypress (q or Q) to end scroll-mode.
 }
 
 // Widget is a widget that hosts a terminal-based application. The user provides the
@@ -131,6 +162,11 @@ type Widget struct {
 	hotKeyDownTime      time.Time
 	hotKeyTimer         *time.Timer
 	isScrolling         bool
+	paste               bool
+	hold                *holder.Widget  // used if scrollbar is enabled
+	cols                *columns.Widget // used if scrollbar is enabled
+	sbar                *vscroll.Widget // used if scrollbar is enabled
+	scrollbarTmpOff     bool            // a simple hack to help with UserInput and Render
 	Callbacks           *gowid.Callbacks
 	gowid.IsSelectable
 }
@@ -175,17 +211,53 @@ func NewExt(opts Options) (*Widget, error) {
 		opts.HotKey = HotKey{tcell.KeyCtrlB}
 	}
 
+	if opts.Scrollback <= 0 {
+		opts.Scrollbar = false
+	}
+
+	var persistence IHotKeyPersistence
+	if opts.HotKeyPersistence != nil {
+		persistence = opts.HotKeyPersistence
+	} else {
+		persistence = &HotKeyDuration{
+			D: 2 * time.Second,
+		}
+	}
+
+	// Always allocate so the scrollbar can be turned on later
+	sbar := vscroll.NewExt(vscroll.VerticalScrollbarUnicodeRunes)
+
+	hold := holder.New(null.New())
+
+	cols := columns.New([]gowid.IContainerWidget{
+		&gowid.ContainerWidget{hold, gowid.RenderWithWeight{W: 1}},
+		&gowid.ContainerWidget{sbar, gowid.RenderWithUnits{U: 1}},
+	})
+
 	res := &Widget{
 		params:             opts,
 		IHotKeyProvider:    opts.HotKey,
-		IHotKeyPersistence: opts.HotKeyPersistence,
+		IHotKeyPersistence: persistence,
 		terminfo:           ti,
+		sbar:               sbar,
+		cols:               cols,
+		hold:               hold,
 		Callbacks:          gowid.NewCallbacks(),
 	}
 
+	res.hold.SetSubWidget(res, nil)
+	res.cols.SetFocus(nil, 0)
+
+	sbar.OnClickAbove(gowid.WidgetCallback{"cb", res.clickUp})
+	sbar.OnClickBelow(gowid.WidgetCallback{"cb", res.clickDown})
+	sbar.OnClickUpArrow(gowid.WidgetCallback{"cb", res.clickUpArrow})
+	sbar.OnClickDownArrow(gowid.WidgetCallback{"cb", res.clickDownArrow})
+
 	var _ gowid.IWidget = res
 	var _ ITerminal = res
 	var _ IWidget = res
+	var _ IHotKeyFunctions = res
+	var _ IScrollbar = res
 	var _ io.Writer = res
 
 	return res, nil
@@ -207,6 +279,22 @@ func (w *Widget) Terminfo() *terminfo.Terminfo {
 	return w.terminfo
 }
 
+func (w *Widget) ScrollbarEnabled() bool {
+	return w.params.Scrollbar
+}
+
+func (w *Widget) EnableScrollbar(app gowid.IApp) {
+	w.params.Scrollbar = true
+}
+
+func (w *Widget) DisableScrollbar(app gowid.IApp) {
+	w.params.Scrollbar = false
+}
+
+func (w *Widget) HotKeyFunctions() []HotKeyInputFn {
+	return w.params.HotKeyFns
+}
+
 func (w *Widget) Bell(app gowid.IApp) {
 	gowid.RunWidgetCallbacks(w.Callbacks, Bell{}, app, w)
 }
@@ -253,6 +341,21 @@ func (w *Widget) RemoveOnBell(f gowid.IIdentity) {
 	gowid.RemoveWidgetCallback(w.Callbacks, Bell{}, f)
 }
 
+func (w *Widget) OnHotKey(f gowid.IWidgetChangedCallback) {
+	gowid.AddWidgetCallback(w.Callbacks, HotKeyCB{}, f)
+}
+
+func (w *Widget) RemoveOnHotKey(f gowid.IIdentity) {
+	gowid.RemoveWidgetCallback(w.Callbacks, HotKeyCB{}, f)
+}
+
+func (w *Widget) PasteState(b ...bool) bool {
+	if len(b) > 0 {
+		w.paste = b[0]
+	}
+	return w.paste
+}
+
 func (w *Widget) HotKeyActive() bool {
 	return w.hotKeyDown
 }
@@ -264,11 +367,14 @@ func (w *Widget) SetHotKeyActive(app gowid.IApp, down bool) {
 		w.hotKeyTimer.Stop()
 	}
 
+	gowid.RunWidgetCallbacks(w.Callbacks, HotKeyCB{}, app, w)
+
 	if down {
 		w.hotKeyDownTime = time.Now()
 		w.hotKeyTimer = time.AfterFunc(w.HotKeyDuration(), func() {
 			app.Run(gowid.RunFunction(func(app gowid.IApp) {
 				w.SetHotKeyActive(app, false)
+				gowid.RunWidgetCallbacks(w.Callbacks, HotKeyCB{}, app, w)
 			}))
 		})
 	}
@@ -284,9 +390,18 @@ func (w *Widget) Scroll(dir ScrollDir, page bool, lines int) {
 	} else {
 		lines = w.canvas.ScrollBuffer(dir, false, gwutil.SomeInt(lines))
 	}
-	// Scrolling is now true if it (a) was previously, or (b) wasn't, but we
-	// scrolled more than one line
-	w.isScrolling = w.isScrolling || lines != 0
+
+	wasScrolling := w.isScrolling
+	if lines != 0 {
+		w.isScrolling = true
+	} else if !w.params.KeyPressToEndScrollMode && dir == ScrollDown {
+		// Disable scroll if we are at the bottom and we tried to scroll down
+		// Thanks @Peter2121 !
+		w.isScrolling = false
+	}
+	if wasScrolling && !w.isScrolling {
+		w.ResetScroll()
+	}
 }
 
 func (w *Widget) ResetScroll() {
@@ -323,6 +438,13 @@ func (w *Widget) Write(p []byte) (n int, err error) {
 }
 
 func (w *Widget) UserInput(ev interface{}, size gowid.IRenderSize, focus gowid.Selector, app gowid.IApp) bool {
+	if !w.scrollbarTmpOff && w.params.Scrollbar {
+		w.scrollbarTmpOff = true
+		res := w.cols.UserInput(ev, size, focus, app)
+		w.scrollbarTmpOff = false
+		w.cols.SetFocus(app, 0)
+		return res
+	}
 	return UserInput(w, ev, size, focus, app)
 }
 
@@ -332,8 +454,19 @@ func (w *Widget) Render(size gowid.IRenderSize, focus gowid.Selector, app gowid.
 		panic(gowid.WidgetSizeError{Widget: w, Size: size, Required: "gowid.IRenderBox"})
 	}
 
+	if !w.scrollbarTmpOff && w.params.Scrollbar {
+		w.scrollbarTmpOff = true
+		c := w.cols.Render(size, focus, app)
+		w.scrollbarTmpOff = false
+		return c
+	}
+
 	w.TouchTerminal(box.BoxColumns(), box.BoxRows(), app)
 
+	w.sbar.Top = w.canvas.Offset
+	w.sbar.Middle = w.canvas.scrollRegionEnd
+	w.sbar.Bottom = gwutil.Max(0, w.canvas.ViewPortCanvas.Canvas.BoxRows()-(box.BoxRows()+w.canvas.Offset))
+
 	return w.canvas
 }
 
@@ -485,6 +618,14 @@ func (w *Widget) StartCommand(app gowid.IApp, width, height int) error {
 		}))
 	}})
 
+	if w.params.EnableBracketedPaste {
+		app.Run(gowid.RunFunction(func(app gowid.IApp) {
+			for _, b := range enablePaste(w.terminfo) {
+				canvas.ProcessByte(b)
+			}
+		}))
+	}
+
 	go func() {
 		for {
 			data := make([]byte, 4096)
@@ -521,6 +662,22 @@ func (w *Widget) StopCommand() {
 	}
 }
 
+func (w *Widget) clickUp(app gowid.IApp, w2 gowid.IWidget) {
+	w.Scroll(ScrollUp, true, 1)
+}
+
+func (w *Widget) clickDown(app gowid.IApp, w2 gowid.IWidget) {
+	w.Scroll(ScrollDown, true, 1)
+}
+
+func (w *Widget) clickUpArrow(app gowid.IApp, w2 gowid.IWidget) {
+	w.Scroll(ScrollUp, false, 1)
+}
+
+func (w *Widget) clickDownArrow(app gowid.IApp, w2 gowid.IWidget) {
+	w.Scroll(ScrollDown, false, 1)
+}
+
 //''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
 
 func UserInput(w IWidget, ev interface{}, size gowid.IRenderSize, focus gowid.Selector, app gowid.IApp) bool {
@@ -558,25 +715,37 @@ func UserInput(w IWidget, ev interface{}, size gowid.IRenderSize, focus gowid.Se
 			// pressed) then the input will not go to the terminal - it's hotkey
 			// function processing.
 			passToTerminal = false
-			res = true
+			res = false
 			deactivate := false
-			switch evk.Key() {
-			case w.HotKey():
-				deactivate = true
-			case tcell.KeyPgUp:
-				w.Scroll(ScrollUp, true, 0)
-				deactivate = true
-			case tcell.KeyPgDn:
-				w.Scroll(ScrollDown, true, 0)
-				deactivate = true
-			case tcell.KeyUp:
-				w.Scroll(ScrollUp, false, 1)
-				deactivate = true
-			case tcell.KeyDown:
-				w.Scroll(ScrollDown, false, 1)
-				deactivate = true
-			default:
-				res = false
+			if whk, ok := w.(IHotKeyFunctions); ok {
+				for _, fn := range whk.HotKeyFunctions() {
+					res = fn(evk, w, app)
+					if res {
+						deactivate = true
+						break
+					}
+				}
+			}
+			if !res {
+				res = true
+				switch evk.Key() {
+				case w.HotKey():
+					deactivate = true
+				case tcell.KeyPgUp:
+					w.Scroll(ScrollUp, true, 0)
+					deactivate = true
+				case tcell.KeyPgDn:
+					w.Scroll(ScrollDown, true, 0)
+					deactivate = true
+				case tcell.KeyUp:
+					w.Scroll(ScrollUp, false, 1)
+					deactivate = true
+				case tcell.KeyDown:
+					w.Scroll(ScrollDown, false, 1)
+					deactivate = true
+				default:
+					res = false
+				}
 			}
 			if deactivate {
 				w.SetHotKeyActive(app, false)
@@ -598,7 +767,7 @@ func UserInput(w IWidget, ev interface{}, size gowid.IRenderSize, focus gowid.Se
 		}
 	}
 	if passToTerminal {
-		seq, parsed := TCellEventToBytes(ev, w.Modes(), app.GetLastMouseState(), w.Terminfo())
+		seq, parsed := TCellEventToBytes(ev, w.Modes(), app.GetLastMouseState(), w, w.Terminfo())
 
 		if parsed {
 			_, err := w.Write(seq)
